<?php

namespace Drupal\workflow\Plugin\Action;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Action\ConfigurableActionBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workflow\Entity\WorkflowTransition;
use Drupal\workflow\Form\WorkflowTransitionForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Sets an entity to a new or given state.
 */
abstract class WorkflowStateActionBase extends ConfigurableActionBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    return parent::calculateDependencies() + [
      'module' => ['workflow'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    $configuration += [
      'field_name' => '',
      'to_sid' => '',
      'comment' => "New state is set by a triggered Action.",
      'force' => FALSE,
    ];
    return $configuration;
  }

  /**
   * Gets the entity's transition that must be executed.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for which a transition must be fetched.
   *
   * @return \Drupal\workflow\Entity\WorkflowTransitionInterface|null
   *   The Transition, if found, else NULL.
   */
  protected function getTransitionForExecution(EntityInterface $entity) {
    $config = $this->configuration;
    $field_name = $config['field_name'];
    $field_name = workflow_get_field_name($entity, $field_name);
    $to_sid = $config['to_sid'];
    $comment = $config['comment'];
    $force = $config['force'];
    $user = workflow_current_user();

    if (!$entity) {
      \Drupal::logger('workflow_action')->notice('Unable to get current entity - entity is not defined.', []);
      return NULL;
    }

    // Get the entity type and numeric ID.
    $entity_id = $entity->id();
    if (!$entity_id) {
      \Drupal::logger('workflow_action')->notice("Unable to get entity $entity_id - entity is not yet saved.", []);
      return NULL;
    }

    if (!$entity->hasField($field_name)) {
      \Drupal::logger('workflow_action')->notice("Unable to process entity $entity_id - entity does not have field $field_name.", []);
      return NULL;
    }

    // In 'after saving new content', the node is already saved. Avoid second insert.
    // @todo Clone?
    $entity->enforceIsNew(FALSE);

    // Translate the Comment. Parse the $comment variables.
    $comment = $this->t($comment, [
      '%title' => $entity->label(),
      // "@" and "%" will automatically run check_plain().
      '%state' => workflow_get_sid_name($to_sid),
      '%user' => $user->getDisplayName(),
    ]);

    // Transition may be not created if entity has not this field.
    $transition = WorkflowTransition::create([
      'entity' => $entity,
      'field_name' => $field_name,
    ])
      ->setValues($to_sid, $user->id(), NULL, $comment)
      ->force($force);

    return $transition;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $element = [];

    // D11: On a VBO view, action form, simple action 'New', $config is empty.
    $config = $this->configuration;
    $field_name = $config['field_name'];
    $to_sid = $config['to_sid'];
    $comment = $config['comment'];
    $force = $config['force'];

    // Create a dummy Transition with config data.
    // @todo Support also other entity types then 'node'.
    $entity_type_id = 'node';
    $entity = $this->getDummyEntity($entity_type_id, $field_name);
    // $field_name may be empty. Determine one for the sake of $transition.
    $dummy_field_name = workflow_get_field_name($entity, $field_name);
    /** @var \Drupal\workflow\Entity\WorkflowTransitionInterface $transition */
    // $transition = workflow_get_transition($entity, $field_name);
    $transition = WorkflowTransition::create([
      'entity' => $entity,
      'field_name' => $dummy_field_name,
      // 'wid' => $wid,
      'to_sid' => $to_sid,
    ])
      // Reset the empty $field_name, if needed.
      ->set('field_name', $field_name)
      ->setComment($comment)
      ->force($force);

    // Add the WorkflowTransitionForm element to the page. How?
    // Set/reset 'options' to Avoid Action Buttons, because that
    // removes the options box&more. No Buttons in config screens!
    // $workflow = $transition->getWorkflow();
    // $workflow->setSetting('options', 'select');
    // Option 1: call transitionElement() directly.
    // $element['#default_value'] = $transition;
    // $element = WorkflowTransitionElement::transitionElement($element, $form_state, $form);
    //
    // Option 2: call WorkflowTransitionForm() directly.
    // @todo Use Form/Widget, instead of transitionElement().
    $workflow_form = WorkflowTransitionForm::createInstance($entity, $field_name, [], $transition);
    $element = WorkflowTransitionForm::trimWorkflowTransitionForm($workflow_form);
    //
    // Option 3: call WorkflowDefaultWidget via NodeFormDisplay.
    // $element = WorkflowDefaultWidget::createInstance($transition);
    // Fetch the element from the widget.
    // @todo #3465257 Following is empty with eca.
    // $element = $element[$field_name]['widget'][0] ?? [];
    // Make adaptations for VBO-form.
    //    $element = [
    //      '#type' => 'details',
    //      '#collapsible' => TRUE,
    //      '#open' => TRUE,
    //    ] + $element;
    //    unset($element['#action']);
    //    //   Remove langcode to avoid error upon submit.
    //    //   InvalidArgumentException: The configuration property langcode.0 doesn't exist.
    //    unset($element['langcode']);
    //
    $element['field_name']['#access'] = TRUE;
    $element['field_name']['widget']['#access'] = TRUE;
    $element['field_name']['widget']['#options']
      = ['' => '-- Any --']
      + $element['field_name']['widget']['#options'];
    unset($element['field_name']['widget']['#options']['_none']);
    $element['force']['#access'] = TRUE;
    $element['to_sid']['#access'] = TRUE;
    $element['to_sid']['widget']['#access'] = TRUE;
    $element['to_sid']['#description'] = $this->t('Please select the state that should be assigned when this action runs.');
    $element['comment']['#title'] = $this->t('Message');
    $element['comment']['#description'] = $this->t(
      'This message will be written
      into the workflow history log when the action runs.
      You may include the following variables: %state, %title, %user.'
    );

    $form['workflow_transition_action_config'] = $element;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {

    // When using Widget/Form, read $input.
    $values = $form_state->getUserInput();
    // When using Element, read $values.
    // $values = $form_state->getValue('workflow_transition_action_config');
    // $values = $form_state->getValues();
    // @todo Use WorkflowDefaultWidget::massage/extractFormValues();
    // @todo Use copyFormValuesToTransition();
    $configuration = [
      'field_name' => $values['field_name'],
      'to_sid' => $values['to_sid'] ?? '',
      'comment' => $values['comment'][0]['value'],
      'force' => $values['force'] ?? FALSE,
    ];
    $this->configuration = $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, ?AccountInterface $account = NULL, $return_as_object = FALSE) {
    $access = AccessResult::allowed();
    return $return_as_object ? $access : $access->isAllowed();
  }

  /**
   * Build a dummy entity for the Action Configuration form.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $field_name
   *   Optional. Can be '', if you want to load any field.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   A dummy entity, just to feed into the next function.
   */
  protected function getDummyEntity($entity_type_id, $field_name) {
    if (!$field_name) {
      $field_map = workflow_get_workflow_fields_by_entity_type($entity_type_id);
      // Get the field name of the (arbitrary) first entity type.
      $field_name = key($field_map);
      if (!$field_name) {
        // We are in problem.
      }
    }
    if ($field_name) {
      $fields = _workflow_info_fields(NULL, $entity_type_id, '', $field_name);
      $field_config = reset($fields);
      $bundles = $field_config->getBundles();
      $entity_bundle = reset($bundles);
      // $wid = $field_config?->getSetting('workflow_type') ?? '';
      // $state = $to_sid ? WorkflowState::load($to_sid) : NULL;
      // // If user has changed field name, then reset the state.
      // if ($wid <> $state?->getWorkflowId()) {
      // $workflow = Workflow::load($wid);
      // $to_sid = $workflow->getCreationSid();
      // }
    }
    $entity_type_manager = \Drupal::service('entity_type.manager');
    $entity = $entity_type_manager->getStorage($entity_type_id)
      ->create(['type' => $entity_bundle]);

    return $entity;
  }

}
